@@ -109,6 +109,11 @@ class Dataset(DatasetMeta):
         self._last_data_clean_status = last_data_clean_status
         self._attachment_url = attachment_url
 
+        self.iter_batch_size = 128
+        self.iter_current_start = 0
+        self.iter_current_end = 0
+        self.iter_current_object = []
+
     def create_version(
             self,
             inherited_version_number: Optional[int] = None,
@@ -217,6 +222,28 @@ class Dataset(DatasetMeta):
                                               self._last_data_clean_status, self._data_count,
                                               self._annotation_status, self._comment))
 
+    def __len__(self):
+        return self._data_count
+
+    def __getitem__(self, index):
+        if index < 0:
+            index = self._data_count + index
+        if index >= self._data_count:
+            raise IndexError
+        if index >= self.iter_current_start and index < self.iter_current_end:
+            return self.iter_current_object[index - self.iter_current_start]
+        try:
+            response = self._x_api.list_objects(self._id, index, self.iter_batch_size)
+            items = response["data"]
+        except IndexError as error:
+            raise ResourceNotExistError(resource="dataset", identification=self._id) from error
+
+        self.iter_current_start = index
+        self.iter_current_end = index + len(items)
+        self.iter_current_object = items
+
+        return self.iter_current_object[index - self.iter_current_start]
+
     def import_data(
             self,
             file_path: str,
@@ -510,7 +537,7 @@ class Dataset(DatasetMeta):
     @staticmethod
     def _download_file(
             **kwargs
-    ) -> (str, Item):
+    ) -> Tuple[str, Item]:
         key, file_name, obj_url = Dataset._get_download_info(
             kwargs["url"],
             kwargs["file_path"],
@@ -534,7 +561,7 @@ class Dataset(DatasetMeta):
             url: str,
             file_path: str,
             image_path: str
-    ) -> (str, str, str):
+    ) -> Tuple[str, str, str]:
         _, key = get_bucket_and_key(url)
 
         if not image_path:
